@@ -1,7 +1,7 @@
 use ecow::EcoString;
 
 use crate::{
-    ast::{Import, SrcSpan, UnqualifiedImport},
+    ast::{SrcSpan, UnqualifiedImport, UntypedImport},
     build::Origin,
     type_::{
         EntityKind, Environment, Error, ModuleInterface, Problems, UnusedModuleAlias,
@@ -9,6 +9,8 @@ use crate::{
     },
 };
 
+use super::Imported;
+
 #[derive(Debug)]
 pub struct Importer<'context, 'problems> {
     origin: Origin,
@@ -32,7 +34,7 @@ impl<'context, 'problems> Importer<'context, 'problems> {
     pub fn run<'code>(
         origin: Origin,
         env: Environment<'context>,
-        imports: &'code [Import<()>],
+        imports: &'code [UntypedImport],
         problems: &'problems mut Problems,
     ) -> Environment<'context> {
         let mut importer = Self::new(origin, env, problems);
@@ -42,7 +44,7 @@ impl<'context, 'problems> Importer<'context, 'problems> {
         importer.environment
     }
 
-    fn register_import(&mut self, import: &Import<()>) {
+    fn register_import(&mut self, import: &UntypedImport) {
         let location = import.location;
         let name = import.module.clone();
 
@@ -51,7 +53,7 @@ impl<'context, 'problems> Importer<'context, 'problems> {
             self.problems.error(Error::UnknownModule {
                 location,
                 name: name.clone(),
-                imported_modules: self.environment.imported_modules.keys().cloned().collect(),
+                suggestions: self.environment.suggest_modules(&name, Imported::Module),
             });
             return;
         };
@@ -117,6 +119,17 @@ impl<'context, 'problems> Importer<'context, 'problems> {
         // Register the unqualified import if it is a value
         let variant = match module.get_public_value(import_name) {
             Some(value) => {
+                let implementations = value.variant.implementations();
+                // Check the target support of the imported value
+                if self.environment.target_support.is_enforced()
+                    && !implementations.supports(self.environment.target)
+                {
+                    self.problems.error(Error::UnsupportedExpressionTarget {
+                        target: self.environment.target,
+                        location,
+                    })
+                }
+
                 self.environment.insert_variable(
                     used_name.clone(),
                     value.variant.clone(),
@@ -139,12 +152,19 @@ impl<'context, 'problems> Importer<'context, 'problems> {
         };
 
         match variant {
-            &ValueConstructorVariant::Record { .. } => self.environment.init_usage(
-                used_name.clone(),
-                EntityKind::ImportedConstructor,
-                location,
-                self.problems,
-            ),
+            ValueConstructorVariant::Record { name, module, .. } => {
+                self.environment.init_usage(
+                    used_name.clone(),
+                    EntityKind::ImportedConstructor,
+                    location,
+                    self.problems,
+                );
+                self.environment.value_names.named_constructor_in_scope(
+                    module.clone(),
+                    name.clone(),
+                    used_name.clone(),
+                );
+            }
             _ => self.environment.init_usage(
                 used_name.clone(),
                 EntityKind::ImportedValue,
@@ -189,7 +209,7 @@ impl<'context, 'problems> Importer<'context, 'problems> {
 
     fn register_module(
         &mut self,
-        import: &Import<()>,
+        import: &UntypedImport,
         import_info: &'context ModuleInterface,
     ) -> Result<(), Error> {
         if let Some(used_name) = import.used_name() {
@@ -225,7 +245,11 @@ impl<'context, 'problems> Importer<'context, 'problems> {
             let _ = self
                 .environment
                 .imported_modules
-                .insert(used_name, (import.location, import_info));
+                .insert(used_name.clone(), (import.location, import_info));
+
+            self.environment
+                .value_names
+                .imported_module(import.module.clone(), used_name)
         };
 
         Ok(())
