@@ -29,16 +29,16 @@ pub enum Constant<T, RecordTag> {
     List {
         location: SrcSpan,
         elements: Vec<Self>,
-        typ: T,
+        type_: T,
     },
 
     Record {
         location: SrcSpan,
-        module: Option<EcoString>,
+        module: Option<(EcoString, SrcSpan)>,
         name: EcoString,
         args: Vec<CallArg<Self>>,
         tag: RecordTag,
-        typ: T,
+        type_: T,
         field_map: Option<FieldMap>,
     },
 
@@ -49,10 +49,10 @@ pub enum Constant<T, RecordTag> {
 
     Var {
         location: SrcSpan,
-        module: Option<EcoString>,
+        module: Option<(EcoString, SrcSpan)>,
         name: EcoString,
         constructor: Option<Box<ValueConstructor>>,
-        typ: T,
+        type_: T,
     },
 
     StringConcatenation {
@@ -65,7 +65,7 @@ pub enum Constant<T, RecordTag> {
     /// even when there are type errors. Should never end up in generated code.
     Invalid {
         location: SrcSpan,
-        typ: T,
+        type_: T,
     },
 }
 
@@ -79,10 +79,10 @@ impl TypedConstant {
             Constant::Tuple { elements, .. } => {
                 type_::tuple(elements.iter().map(|e| e.type_()).collect())
             }
-            Constant::List { typ, .. }
-            | Constant::Record { typ, .. }
-            | Constant::Var { typ, .. }
-            | Constant::Invalid { typ, .. } => typ.clone(),
+            Constant::List { type_, .. }
+            | Constant::Record { type_, .. }
+            | Constant::Var { type_, .. }
+            | Constant::Invalid { type_, .. } => type_.clone(),
         }
     }
 }
