@@ -86,6 +86,20 @@ impl Target {
     }
 }
 
+#[derive(Debug, PartialEq, Eq, Clone, Copy)]
+/// This is used to skip compiling the root package when running the main
+/// function coming from a dependency. This way a dependency can be run even
+/// there's compilation errors in the root package.
+///
+pub enum Compile {
+    /// The default compiler behaviour, compile all packages.
+    ///
+    All,
+    /// Only compile the dependency packages, skipping the root package.
+    ///
+    DepsOnly,
+}
+
 #[derive(Debug, PartialEq, Eq, Clone, Copy)]
 pub enum Codegen {
     All,
@@ -310,7 +324,7 @@ pub enum Located<'a> {
     Pattern(&'a TypedPattern),
     PatternSpread {
         spread_location: SrcSpan,
-        arguments: &'a Vec<CallArg<Pattern<Arc<Type>>>>,
+        arguments: &'a Vec<CallArg<TypedPattern>>,
     },
     Statement(&'a TypedStatement),
     Expression(&'a TypedExpr),
