@@ -22,7 +22,7 @@ fn bit_array_float() {
         r#"pub fn main() {
   let b = 16
   let floats = <<1.0:16-float, 5.0:float-32, 6.0:float-64-little, 1.0:float-size(b)>>
-  let assert <<1.0:16-float, 5.0:float-32, 6.0:float-64-little, 1.0:float-size(b)>> = floats 
+  let assert <<1.0:16-float, 5.0:float-32, 6.0:float-64-little, 1.0:float-size(b)>> = floats
 }"#
     );
 }
@@ -161,3 +161,46 @@ fn unicode_bit_array_2() {
 }"#
     );
 }
+
+#[test]
+fn bit_array_literal_string_constant_is_treated_as_utf8() {
+    assert_erl!(
+        r#"
+const a = <<"hello", " ", "world">>
+pub fn main() { a }
+"#
+    );
+}
+
+#[test]
+fn bit_array_literal_string_is_treated_as_utf8() {
+    assert_erl!(
+        r#"
+pub fn main() {
+  <<"hello", " ", "world">>
+}"#
+    );
+}
+
+#[test]
+fn bit_array_literal_string_pattern_is_treated_as_utf8() {
+    assert_erl!(
+        r#"
+pub fn main() {
+  case <<>> {
+    <<"a", "b", _:bits>> -> 1
+    _ -> 2
+  }
+}"#
+    );
+}
+
+#[test]
+fn discard_utf8_pattern() {
+    assert_erl!(
+        r#"
+pub fn main() {
+    let assert <<_:utf8, rest:bits>> = <<>>
+}"#
+    );
+}
