@@ -1,7 +1,6 @@
-use super::{Constructor, Decision, Match, Variable};
+use super::{printer::Printer, Constructor, Decision, Match, Variable};
 use crate::type_::environment::Environment;
 use ecow::EcoString;
-use itertools::Itertools;
 use std::collections::{HashMap, HashSet};
 
 /// Returns a list of patterns not covered by the match expression.
@@ -9,7 +8,13 @@ pub fn missing_patterns(matches: &Match, environment: &Environment<'_>) -> Vec<E
     let mut names = HashSet::new();
     let mut steps = Vec::new();
 
-    add_missing_patterns(&matches.tree, &mut steps, &mut names, environment);
+    add_missing_patterns(
+        &matches.tree,
+        &matches.subject_variables,
+        &mut steps,
+        &mut names,
+        environment,
+    );
 
     let mut missing: Vec<EcoString> = names.into_iter().collect();
 
@@ -21,10 +26,15 @@ pub fn missing_patterns(matches: &Match, environment: &Environment<'_>) -> Vec<E
 /// Information about a single constructor/value (aka term) being tested, used
 /// to build a list of names of missing patterns.
 #[derive(Debug)]
-enum Term {
+pub enum Term {
     Variant {
         variable: Variable,
         name: EcoString,
+        module: EcoString,
+        arguments: Vec<Variable>,
+    },
+    Tuple {
+        variable: Variable,
         arguments: Vec<Variable>,
     },
     Infinite {
@@ -41,86 +51,20 @@ enum Term {
 }
 
 impl Term {
-    fn variable(&self) -> &Variable {
+    pub fn variable(&self) -> &Variable {
         match self {
             Term::Variant { variable, .. } => variable,
+            Term::Tuple { variable, .. } => variable,
             Term::Infinite { variable } => variable,
             Term::EmptyList { variable } => variable,
             Term::List { variable, .. } => variable,
         }
     }
-
-    fn pattern_string(&self, terms: &[Term], mapping: &HashMap<usize, usize>) -> EcoString {
-        match self {
-            Term::Variant {
-                name, arguments, ..
-            } => {
-                if arguments.is_empty() {
-                    return name.clone();
-                }
-                let args = arguments
-                    .iter()
-                    .map(|variable| {
-                        mapping
-                            .get(&variable.id)
-                            .map(|&idx| {
-                                terms
-                                    .get(idx)
-                                    .expect("Term must exist")
-                                    .pattern_string(terms, mapping)
-                            })
-                            .unwrap_or_else(|| "_".into())
-                    })
-                    .join(", ");
-                format!("{}({})", name, args).into()
-            }
-
-            Term::Infinite { .. } => "_".into(),
-
-            Term::EmptyList { .. } => "[]".into(),
-
-            Term::List { .. } => format!("[{}]", self.list_pattern_string(terms, mapping)).into(),
-        }
-    }
-
-    fn list_pattern_string(&self, terms: &[Term], mapping: &HashMap<usize, usize>) -> EcoString {
-        match self {
-            Term::Infinite { .. } | Term::Variant { .. } => "_".into(),
-
-            Term::EmptyList { .. } => "".into(),
-
-            Term::List { first, rest, .. } => {
-                let first = mapping
-                    .get(&first.id)
-                    .map(|&idx| {
-                        terms
-                            .get(idx)
-                            .expect("Term must exist")
-                            .pattern_string(terms, mapping)
-                    })
-                    .unwrap_or_else(|| "_".into());
-                let rest = mapping
-                    .get(&rest.id)
-                    .map(|&idx| {
-                        terms
-                            .get(idx)
-                            .expect("Term must exist")
-                            .list_pattern_string(terms, mapping)
-                    })
-                    .unwrap_or_else(|| "_".into());
-
-                match rest.as_str() {
-                    "" => first,
-                    "_" => format!("{first}, ..").into(),
-                    _ => format!("{first}, {rest}").into(),
-                }
-            }
-        }
-    }
 }
 
 fn add_missing_patterns(
     node: &Decision,
+    subjects: &Vec<Variable>,
     terms: &mut Vec<Term>,
     missing: &mut HashSet<EcoString>,
     environment: &Environment<'_>,
@@ -130,6 +74,7 @@ fn add_missing_patterns(
 
         Decision::Failure => {
             let mut mapping = HashMap::new();
+            let printer = Printer::new(&environment.value_names);
 
             // At this point the terms stack looks something like this:
             // `[term, term + arguments, term, ...]`. To construct a pattern
@@ -148,16 +93,13 @@ fn add_missing_patterns(
                 _ = mapping.insert(step.variable().id, index);
             }
 
-            let name = terms
-                .first()
-                .map(|term| term.pattern_string(terms, &mapping))
-                .unwrap_or_else(|| "_".into());
+            let pattern = printer.print_terms(subjects, terms, &mapping);
 
-            _ = missing.insert(name);
+            _ = missing.insert(pattern);
         }
 
         Decision::Guard(_, _, fallback) => {
-            add_missing_patterns(fallback, terms, missing, environment);
+            add_missing_patterns(fallback, subjects, terms, missing, environment);
         }
 
         Decision::Switch(variable, cases, fallback) => {
@@ -175,9 +117,8 @@ fn add_missing_patterns(
 
                     Constructor::Tuple(_) => {
                         let arguments = case.arguments.clone();
-                        terms.push(Term::Variant {
+                        terms.push(Term::Tuple {
                             variable: variable.clone(),
-                            name: "#".into(),
                             arguments,
                         });
                     }
@@ -196,17 +137,18 @@ fn add_missing_patterns(
                         terms.push(Term::Variant {
                             variable: variable.clone(),
                             name,
+                            module,
                             arguments: case.arguments.clone(),
                         });
                     }
                 }
 
-                add_missing_patterns(&case.body, terms, missing, environment);
+                add_missing_patterns(&case.body, subjects, terms, missing, environment);
                 _ = terms.pop();
             }
 
             if let Some(node) = fallback {
-                add_missing_patterns(node, terms, missing, environment);
+                add_missing_patterns(node, subjects, terms, missing, environment);
             }
         }
 
@@ -218,7 +160,7 @@ fn add_missing_patterns(
             terms.push(Term::EmptyList {
                 variable: variable.clone(),
             });
-            add_missing_patterns(empty, terms, missing, environment);
+            add_missing_patterns(empty, subjects, terms, missing, environment);
             _ = terms.pop();
 
             terms.push(Term::List {
@@ -226,7 +168,7 @@ fn add_missing_patterns(
                 first: non_empty.first.clone(),
                 rest: non_empty.rest.clone(),
             });
-            add_missing_patterns(&non_empty.decision, terms, missing, environment);
+            add_missing_patterns(&non_empty.decision, subjects, terms, missing, environment);
             _ = terms.pop();
         }
     }
