@@ -78,6 +78,8 @@ const REMOVE_REDUNDANT_TUPLES: &str = "Remove redundant tuples";
 const CONVERT_TO_CASE: &str = "Convert to case";
 const USE_LABEL_SHORTHAND_SYNTAX: &str = "Use label shorthand syntax";
 const FILL_LABELS: &str = "Fill labels";
+const ASSIGN_UNUSED_RESULT: &str = "Assign unused Result value to `_`";
+const ADD_MISSING_PATTERNS: &str = "Add missing patterns";
 
 macro_rules! assert_code_action {
     ($title:expr, $code:literal, $range:expr $(,)?) => {
@@ -1123,6 +1125,441 @@ pub fn main() {
     );
 }
 
+#[test]
+fn test_assign_unused_result() {
+    assert_code_action!(
+        ASSIGN_UNUSED_RESULT,
+        r#"
+pub fn main() {
+    Ok(0)
+    Nil
+}
+"#,
+        find_position_of("Ok").select_until(find_position_of("(0)")),
+    );
+}
+
+#[test]
+fn test_assign_unused_result_in_block() {
+    assert_code_action!(
+        ASSIGN_UNUSED_RESULT,
+        r#"
+pub fn main() {
+    {
+        Ok(0)
+        Nil
+    }
+    Nil
+}
+"#,
+        find_position_of("Ok").select_until(find_position_of("(0)")),
+    );
+}
+
+#[test]
+fn test_assign_unused_result_on_block_start() {
+    assert_code_action!(
+        ASSIGN_UNUSED_RESULT,
+        r#"
+pub fn main() {
+    {
+        Ok(0)
+        Ok(0)
+    }
+    Nil
+}
+"#,
+        find_position_of("{").nth_occurrence(2).to_selection()
+    );
+}
+
+#[test]
+fn test_assign_unused_result_on_block_end() {
+    assert_code_action!(
+        ASSIGN_UNUSED_RESULT,
+        r#"
+pub fn main() {
+    {
+        Ok(0)
+        Ok(0)
+    }
+    Nil
+}
+"#,
+        find_position_of("}").to_selection()
+    );
+}
+
+#[test]
+#[should_panic(expected = "No action with the given title")]
+fn test_assign_unused_result_inside_block() {
+    assert_code_action!(
+        ASSIGN_UNUSED_RESULT,
+        r#"
+pub fn main() {
+    {
+        Nil
+        Ok(1)
+    }
+}
+"#,
+        find_position_of("Ok").select_until(find_position_of("(1)"))
+    );
+}
+
+#[test]
+fn test_assign_unused_result_only_first_action() {
+    assert_code_action!(
+        ASSIGN_UNUSED_RESULT,
+        r#"
+pub fn main() {
+    Ok(0)
+    Ok(1)
+    Nil
+}
+"#,
+        find_position_of("Ok").select_until(find_position_of("(0)"))
+    );
+}
+
+#[test]
+#[should_panic(expected = "No action with the given title")]
+fn test_assign_unused_result_not_on_return_value() {
+    assert_code_action!(
+        ASSIGN_UNUSED_RESULT,
+        r#"
+pub fn main() {
+    Ok(0)
+}
+"#,
+        find_position_of("Ok").select_until(find_position_of("(0)"))
+    );
+}
+
+#[test]
+#[should_panic(expected = "No action with the given title")]
+fn test_assign_unused_result_not_on_return_value_in_block() {
+    assert_code_action!(
+        ASSIGN_UNUSED_RESULT,
+        r#"
+pub fn main() {
+    let _ = {
+        Ok(0)
+    }
+    Nil
+}"#,
+        find_position_of("Ok").select_until(find_position_of("(0)"))
+    );
+}
+
+#[test]
+fn test_import_module_from_function() {
+    let src = "
+pub fn main() {
+  result.is_ok()
+}
+";
+
+    assert_code_action!(
+        "Import `result`",
+        TestProject::for_source(src).add_hex_module("result", "pub fn is_ok() {}"),
+        find_position_of("result").select_until(find_position_of("."))
+    );
+}
+
+#[test]
+fn test_import_path_module_from_function() {
+    let src = r#"
+pub fn main() {
+  io.println("Hello, world!")
+}
+"#;
+
+    assert_code_action!(
+        "Import `gleam/io`",
+        TestProject::for_source(src)
+            .add_hex_module("gleam/io", "pub fn println(message: String) {}"),
+        find_position_of("io").select_until(find_position_of("."))
+    );
+}
+
+#[test]
+fn test_import_module_from_type() {
+    let src = "type Wobble = wibble.Wubble";
+
+    assert_code_action!(
+        "Import `mod/wibble`",
+        TestProject::for_source(src).add_hex_module("mod/wibble", "pub type Wubble { Wubble }"),
+        find_position_of("wibble").select_until(find_position_of("."))
+    );
+}
+
+#[test]
+fn test_import_module_from_constructor() {
+    let src = "
+pub fn main() {
+  let value = values.Value(10)
+}
+";
+
+    assert_code_action!(
+        "Import `values`",
+        TestProject::for_source(src).add_hex_module("values", "pub type Value { Value(Int) }"),
+        find_position_of("values").select_until(find_position_of("."))
+    );
+}
+
+#[test]
+fn test_rename_module_for_imported() {
+    let src = r#"
+import gleam/io
+
+pub fn main() {
+  i.println("Hello, world!")
+}
+"#;
+
+    assert_code_action!(
+        "Did you mean `io`",
+        TestProject::for_source(src)
+            .add_hex_module("gleam/io", "pub fn println(message: String) {}"),
+        find_position_of("i.").select_until(find_position_of("println"))
+    );
+}
+
+#[test]
+fn test_import_similar_module() {
+    let src = "
+pub fn main() {
+  reult.is_ok()
+}
+";
+
+    assert_code_action!(
+        "Import `result`",
+        TestProject::for_source(src).add_hex_module("result", "pub fn is_ok() {}"),
+        find_position_of("reult").select_until(find_position_of("."))
+    );
+}
+
+#[test]
+fn test_no_action_to_import_module_without_value() {
+    // The language server should not suggest a code action
+    // to import a module if it doesn't have a value with
+    // the same name as we are trying to access
+    let src = "
+pub fn main() {
+  io.hello_world()
+}
+";
+
+    let title = "Import `io`";
+
+    assert_no_code_actions!(
+        title,
+        TestProject::for_source(src).add_hex_module("io", "pub fn println() {}"),
+        find_position_of("io").select_until(find_position_of("."))
+    );
+}
+
+#[test]
+fn test_no_action_to_import_module_without_type() {
+    let src = "type Name = int.String";
+
+    let title = "Import `int`";
+
+    assert_no_code_actions!(
+        title,
+        TestProject::for_source(src).add_hex_module("int", ""),
+        find_position_of("int").select_until(find_position_of("."))
+    );
+}
+
+#[test]
+fn test_no_action_to_import_module_with_private_value() {
+    // The language server should not suggest a code action
+    // to import a module if the value we are trying to
+    // access is private.
+    let src = "
+pub fn main() {
+  mod.internal()
+}
+";
+
+    let title = "Import `mod`";
+
+    assert_no_code_actions!(
+        title,
+        TestProject::for_source(src).add_hex_module("mod", "fn internal() {}"),
+        find_position_of("mod").select_until(find_position_of("."))
+    );
+}
+
+#[test]
+fn test_no_action_to_import_module_with_private_type() {
+    let src = "type T = module.T";
+
+    let title = "Import `module`";
+
+    assert_no_code_actions!(
+        title,
+        TestProject::for_source(src).add_hex_module("module", "type T { T }"),
+        find_position_of("module").select_until(find_position_of("."))
+    );
+}
+
+#[test]
+fn test_no_action_to_import_module_with_constructor_named_same_as_type() {
+    let src = "type NotAType = shapes.Rectangle";
+
+    let title = "Import `shapes`";
+
+    assert_no_code_actions!(
+        title,
+        TestProject::for_source(src)
+            .add_hex_module("shapes", "pub type Shape { Rectangle, Circle }"),
+        find_position_of("shapes").select_until(find_position_of("."))
+    );
+}
+
+#[test]
+fn add_missing_patterns_bool() {
+    assert_code_action!(
+        ADD_MISSING_PATTERNS,
+        "
+pub fn main() {
+  let bool = True
+  case bool {}
+}
+",
+        find_position_of("case").select_until(find_position_of("bool {"))
+    );
+}
+
+#[test]
+fn add_missing_patterns_custom_type() {
+    assert_code_action!(
+        ADD_MISSING_PATTERNS,
+        "
+type Wibble {
+  Wibble
+  Wobble
+  Wubble
+}
+
+pub fn main() {
+  let wibble = Wobble
+  case wibble {
+    Wobble -> Nil
+  }
+}
+",
+        find_position_of("case").select_until(find_position_of("wibble {"))
+    );
+}
+
+#[test]
+fn add_missing_patterns_tuple() {
+    assert_code_action!(
+        ADD_MISSING_PATTERNS,
+        "
+pub fn main() {
+  let two_at_once = #(True, Ok(1))
+  case two_at_once {
+    #(False, Error(_)) -> Nil
+  }
+}
+",
+        find_position_of("case").select_until(find_position_of("two_at_once {"))
+    );
+}
+
+#[test]
+fn add_missing_patterns_list() {
+    assert_code_action!(
+        ADD_MISSING_PATTERNS,
+        "
+pub fn main() {
+  let list = [1, 2, 3]
+  case list {
+    [a, b, c, 4 as d] -> d
+  }
+}
+",
+        find_position_of("case").select_until(find_position_of("list {"))
+    );
+}
+
+#[test]
+fn add_missing_patterns_infinite() {
+    assert_code_action!(
+        ADD_MISSING_PATTERNS,
+        r#"
+pub fn main() {
+  let value = 3
+  case value {
+    1 -> "one"
+    2 -> "two"
+    3 -> "three"
+  }
+}
+"#,
+        find_position_of("case").select_until(find_position_of("value {"))
+    );
+}
+
+#[test]
+fn add_missing_patterns_multi() {
+    assert_code_action!(
+        ADD_MISSING_PATTERNS,
+        r#"
+pub fn main() {
+  let a = True
+  let b = 1
+  case a, b {
+
+  }
+}
+"#,
+        find_position_of("case").select_until(find_position_of("b {"))
+    );
+}
+
+#[test]
+fn add_missing_patterns_inline() {
+    // Ensure we correctly detect the indentation, if the case expression
+    // does not start at the beginning of the line
+    assert_code_action!(
+        ADD_MISSING_PATTERNS,
+        r#"
+pub fn main() {
+  let a = True
+  let value = case a {}
+}
+"#,
+        find_position_of("case").select_until(find_position_of("a {"))
+    );
+}
+
+#[test]
+fn import_module_from_pattern() {
+    let src = "
+pub fn main(res) {
+  case res {
+    result.Ok(_) -> Nil
+    result.Error(_) -> Nil
+  }
+}
+";
+
+    assert_code_action!(
+        "Import `result`",
+        TestProject::for_source(src)
+            .add_hex_module("result", "pub type Result(v, e) { Ok(v) Error(e) }"),
+        find_position_of("result").select_until(find_position_of("."))
+    );
+}
+
 /* TODO: implement qualified unused location
 #[test]
 fn test_remove_unused_qualified_action() {
