@@ -28,9 +28,9 @@ macro_rules! assert_completion {
         let src = $project.src;
         let result = completion_with_prefix($project, "");
         let output = format!(
-            "{}\n\n----- Completion content -----\n{:#?}",
+            "{}\n\n----- Completion content -----\n{}",
             show_complete(src, Position::new(0, 0)),
-            result
+            format_completion_results(result)
         );
         insta::assert_snapshot!(insta::internals::AutoName, output, src);
     };
@@ -38,9 +38,9 @@ macro_rules! assert_completion {
         let src = $project.src;
         let result = completion($project, $position);
         let output = format!(
-            "{}\n\n----- Completion content -----\n{:#?}",
+            "{}\n\n----- Completion content -----\n{}",
             show_complete(src, $position),
-            result
+            format_completion_results(result)
         );
         insta::assert_snapshot!(insta::internals::AutoName, output, src);
     };
@@ -53,14 +53,110 @@ macro_rules! assert_completion_with_prefix {
         let result = completion_with_prefix($project, $prefix);
         let line = 1 + $prefix.lines().count();
         let output = format!(
-            "{}\n\n----- Completion content -----\n{:#?}",
+            "{}\n\n----- Completion content -----\n{}",
             show_complete(src, Position::new(line as u32, 0)),
-            result
+            format_completion_results(result)
         );
         insta::assert_snapshot!(insta::internals::AutoName, output, src);
     };
 }
 
+fn format_completion_results(completions: Vec<CompletionItem>) -> EcoString {
+    use std::fmt::Write;
+    let mut buffer: EcoString = "".into();
+
+    for CompletionItem {
+        label,
+        label_details,
+        kind,
+        detail,
+        documentation,
+        deprecated,
+        preselect,
+        sort_text,
+        filter_text,
+        insert_text,
+        insert_text_format,
+        insert_text_mode,
+        text_edit,
+        additional_text_edits,
+        command,
+        commit_characters,
+        data,
+        tags,
+    } in completions
+    {
+        assert!(deprecated.is_none());
+        assert!(preselect.is_none());
+        assert!(filter_text.is_none());
+        assert!(insert_text.is_none());
+        assert!(insert_text_format.is_none());
+        assert!(insert_text_mode.is_none());
+        assert!(command.is_none());
+        assert!(commit_characters.is_none());
+        assert!(data.is_none());
+        assert!(tags.is_none());
+
+        buffer.push_str(&label);
+
+        if let Some(kind) = kind {
+            write!(buffer, "\n  kind:   {:?}", kind).unwrap();
+        }
+
+        if let Some(detail) = detail {
+            write!(buffer, "\n  detail: {}", detail).unwrap();
+        }
+
+        if let Some(sort_text) = sort_text {
+            write!(buffer, "\n  sort:   {}", sort_text).unwrap();
+        }
+
+        if let Some(label_details) = label_details {
+            assert!(label_details.detail.is_none());
+            if let Some(desc) = label_details.description {
+                write!(buffer, "\n  desc:   {}", desc).unwrap();
+            }
+        }
+
+        if let Some(documentation) = documentation {
+            let lsp_types::Documentation::MarkupContent(m) = documentation else {
+                panic!("unexpected docs in test {:?}", documentation);
+            };
+            match m.kind {
+                lsp_types::MarkupKind::Markdown => (),
+                lsp_types::MarkupKind::PlainText => {
+                    panic!("unexpected docs markup kind {:?}", m.kind)
+                }
+            };
+            write!(buffer, "\n  docs:   {:?}", m.value).unwrap();
+        }
+
+        let edit = |buffer: &mut EcoString, e: lsp_types::TextEdit| {
+            let a = e.range.start.line;
+            let b = e.range.start.character;
+            let c = e.range.start.line;
+            let d = e.range.start.character;
+            write!(buffer, "\n    [{a}:{b}-{c}:{d}]: {:?}", e.new_text).unwrap();
+        };
+
+        if let Some(text_edit) = text_edit {
+            let lsp_types::CompletionTextEdit::Edit(e) = text_edit else {
+                panic!("unexpected text edit in test {:?}", text_edit);
+            };
+            buffer.push_str("\n  edits:");
+            edit(&mut buffer, e);
+        }
+
+        for e in additional_text_edits.unwrap_or_default() {
+            edit(&mut buffer, e);
+        }
+
+        buffer.push('\n');
+    }
+
+    buffer
+}
+
 fn completion(tester: TestProject<'_>, position: Position) -> Vec<CompletionItem> {
     tester.at(position, |engine, param, src| {
         let response = engine.completion(param, src);
@@ -712,6 +808,231 @@ pub fn wibble(
     assert_completion!(TestProject::for_source(code), Position::new(4, 0));
 }
 
+#[test]
+fn local_variable() {
+    let code = "
+pub fn main(wibble: Int) {
+  let wobble = 1
+  w
+
+  let wabble = 2
+}
+";
+
+    assert_completion!(TestProject::for_source(code), Position::new(3, 3));
+}
+
+#[test]
+fn local_variable_anonymous_function() {
+    let code = "
+pub fn main() {
+  let add_one = fn(wibble: Int) { wibble + 1 }
+  add_one(1)
+}
+";
+
+    assert_completion!(TestProject::for_source(code), Position::new(2, 40));
+}
+
+#[test]
+fn local_variable_nested_anonymous_function() {
+    let code = "
+pub fn main() {
+  let add_one = fn(wibble: Int) {
+    let wabble = 1
+    let add_two = fn(wobble: Int) { wobble + 2 }
+    wibble + add_two(1)
+  }
+  add_one(1)
+}
+";
+
+    assert_completion!(TestProject::for_source(code), Position::new(4, 42));
+}
+
+#[test]
+fn local_variable_ignore_anonymous_function_args() {
+    let code = "
+pub fn main() {
+  let add_one = fn(wibble: Int) { wibble + 1 }
+  let wobble = 1
+  w
+}
+";
+
+    assert_completion!(TestProject::for_source(code), Position::new(4, 3));
+}
+
+#[test]
+fn local_variable_ignore_anonymous_function_args_nested() {
+    let code = "
+pub fn main() {
+  let add_one = fn(wibble: Int) {
+    let wabble = 1
+    let add_two = fn(wobble: Int) { wobble + 2 }
+    wibble + add_two(1)
+  }
+  add_one(1)
+}
+";
+
+    assert_completion!(TestProject::for_source(code), Position::new(5, 10));
+}
+
+#[test]
+fn local_variable_ignore_anonymous_function_returned() {
+    let code = "
+pub fn main() {
+  fn(wibble: Int) {
+    let wabble = 1
+    let add_two = fn(wobble: Int) { wobble + 2 }
+    wibble + add_two(1)
+  }
+}
+";
+
+    assert_completion!(TestProject::for_source(code), Position::new(5, 10));
+}
+
+#[test]
+fn local_variable_case_expression() {
+    let code = "
+pub fn main() {
+  case True {
+    True as wibble -> { todo }
+    False -> { todo }
+  }
+}
+";
+
+    assert_completion!(TestProject::for_source(code), Position::new(3, 25));
+}
+
+#[test]
+fn local_variable_inside_nested_exprs() {
+    let code = r#"
+type Foo { Bar(List(#(Bool))) }
+fn wibble() {
+  Bar([#(!{
+    let foo = True
+    foo
+  })])
+  todo
+}
+"#;
+
+    assert_completion!(TestProject::for_source(code), Position::new(5, 7));
+}
+
+#[test]
+fn local_variable_pipe() {
+    let code = "
+pub fn main() {
+  let add_one = fn(wibble: Int) { wibble + 1 }
+  let wobble = 1
+  wobble |> add_one
+}
+";
+
+    assert_completion!(TestProject::for_source(code), Position::new(4, 19));
+}
+
+#[test]
+fn local_variable_pipe_with_args() {
+    let code = "
+pub fn main() {
+  let add_one = fn(wibble: Int, wobble: Int) { wibble + wobble }
+  let wobble = 1
+  let wibble = 2
+  wobble |> add_one(1, wibble)
+}
+";
+
+    assert_completion!(TestProject::for_source(code), Position::new(5, 29));
+}
+
+#[test]
+fn local_variable_function_call() {
+    let code = "
+fn add_one(wibble: Int) -> Int {
+  wibble + 1
+}
+
+pub fn main() {
+  let wobble = 1
+  add_one(wobble)
+}
+";
+
+    assert_completion!(TestProject::for_source(code), Position::new(7, 16));
+}
+
+#[test]
+fn local_variable_ignored() {
+    let code = "
+fn wibble() {
+  let a = 1
+  let _b = 2
+
+}
+";
+    assert_completion!(TestProject::for_source(code), Position::new(4, 0));
+}
+
+#[test]
+fn local_variable_as() {
+    let code = "
+fn wibble() {
+  let b as c = 5
+
+}
+";
+    assert_completion!(TestProject::for_source(code), Position::new(3, 0));
+}
+
+#[test]
+fn local_variable_tuple() {
+    let code = "
+fn wibble() {
+  let assert #([d, e] as f, g) = #([0, 1], 2)
+
+}
+";
+    assert_completion!(TestProject::for_source(code), Position::new(3, 0));
+}
+
+#[test]
+fn local_variable_bit_array() {
+    let code = "
+fn wibble() {
+  let assert <<h:1>> as i = <<1:1>>
+
+}
+";
+    assert_completion!(TestProject::for_source(code), Position::new(3, 0));
+}
+
+#[test]
+fn local_variable_string() {
+    let code = r#"
+fn wibble() {
+  let assert "a" <> j = "ab"
+
+}
+"#;
+    assert_completion!(TestProject::for_source(code), Position::new(3, 0));
+}
+
+#[test]
+fn local_variable_ignore_within_function() {
+    let code = "
+fn main(a, b, z) {
+    Nil
+}
+";
+    assert_completion!(TestProject::for_source(code), Position::new(1, 14));
+}
+
 #[test]
 fn internal_values_from_root_package_are_in_the_completions() {
     let dep = r#"
@@ -1383,3 +1704,52 @@ fn fun() {
     );
     assert_eq!(completions, vec![],);
 }
+
+#[test]
+fn completions_for_prelude_values() {
+    let code = "
+pub fn main() {
+  let my_bool = T
+}
+";
+
+    assert_completion!(TestProject::for_source(code), Position::new(2, 17));
+}
+
+#[test]
+fn variable_shadowing() {
+    let code = "
+pub fn main() {
+  let x = 1
+  let x = [1, 2]
+
+}
+";
+
+    assert_completion!(TestProject::for_source(code), Position::new(4, 0));
+}
+
+#[test]
+fn argument_shadowing() {
+    let code = "
+pub fn main(x: Int) {
+  fn(x: Float) {
+
+  }
+}
+";
+
+    assert_completion!(TestProject::for_source(code), Position::new(3, 0));
+}
+
+#[test]
+fn argument_variable_shadowing() {
+    let code = "
+pub fn main(x: Int) {
+  let x = [1, 2]
+
+}
+";
+
+    assert_completion!(TestProject::for_source(code), Position::new(3, 0));
+}
