@@ -14,92 +14,7 @@ fn hover(tester: TestProject<'_>, position: Position) -> Option<Hover> {
     })
 }
 
-struct PositionFinder {
-    value: String,
-    offset: usize,
-    nth_occurrence: usize,
-}
-
-impl PositionFinder {
-    fn with_char_offset(self, offset: usize) -> Self {
-        Self {
-            value: self.value,
-            offset,
-            nth_occurrence: self.nth_occurrence,
-        }
-    }
-
-    fn under_char(self, char: char) -> Self {
-        Self {
-            offset: self.value.find(char).unwrap_or(0),
-            value: self.value,
-            nth_occurrence: self.nth_occurrence,
-        }
-    }
-
-    fn under_last_char(self) -> Self {
-        let len = self.value.len();
-        self.with_char_offset(len - 1)
-    }
-
-    fn nth_occurrence(self, nth_occurrence: usize) -> Self {
-        Self {
-            value: self.value,
-            offset: self.offset,
-            nth_occurrence,
-        }
-    }
-
-    fn for_value(value: &str) -> Self {
-        Self {
-            value: value.into(),
-            offset: 0,
-            nth_occurrence: 1,
-        }
-    }
-
-    fn find_position(&self, src: &str) -> Position {
-        let PositionFinder {
-            value,
-            offset,
-            nth_occurrence,
-        } = self;
-
-        let byte_index = src
-            .match_indices(value)
-            .nth(nth_occurrence - 1)
-            .expect("no match for position")
-            .0;
-
-        byte_index_to_position(src, byte_index + offset)
-    }
-}
-
-fn find_position_of(value: &str) -> PositionFinder {
-    PositionFinder::for_value(value)
-}
-
-fn byte_index_to_position(src: &str, byte_index: usize) -> Position {
-    let mut line = 0;
-    let mut col = 0;
-
-    for (i, char) in src.bytes().enumerate() {
-        if i == byte_index {
-            break;
-        }
-
-        if char == b'\n' {
-            line += 1;
-            col = 0;
-        } else {
-            col += 1;
-        }
-    }
-
-    Position::new(line, col)
-}
-
-fn show_hover(code: &str, range: Range, position: Position) -> String {
+pub fn show_hover(code: &str, range: Range, position: Position) -> String {
     let Range { start, end } = range;
 
     // When we display the over range the end character is always excluded!
@@ -833,3 +748,51 @@ pub fn main() {
         find_position_of("..")
     );
 }
+
+#[test]
+fn hover_label_shorthand_in_call_arg() {
+    assert_hover!(
+        "
+fn wibble(arg1 arg1: Int, arg2 arg2: Bool) { Nil }
+
+fn main() {
+  let arg1 = 1
+  let arg2 = True
+  wibble(arg2:, arg1:)
+}
+",
+        find_position_of("arg2:").nth_occurrence(2)
+    );
+}
+
+#[test]
+fn hover_label_shorthand_in_pattern_call_arg() {
+    assert_hover!(
+        "
+pub type Wibble { Wibble(arg1: Int, arg2: Bool) }
+
+pub fn main() {
+  case todo {
+    Wibble(arg2:, ..) -> todo
+  }
+}
+",
+        find_position_of("arg2:")
+            .nth_occurrence(2)
+            .under_last_char()
+    );
+}
+
+#[test]
+fn hover_label_shorthand_in_pattern_call_arg_2() {
+    assert_hover!(
+        "
+pub type Wibble { Wibble(arg1: Int, arg2: Bool) }
+
+pub fn main() {
+  let Wibble(arg2:, ..) = todo
+}
+",
+        find_position_of("arg2:").nth_occurrence(2).under_char('r')
+    );
+}
