@@ -1,3 +1,4 @@
+use hexpm::version::Version;
 use rand::Rng;
 use type_::{AccessorsMap, FieldMap, RecordAccessor};
 
@@ -37,7 +38,6 @@ fn constant_module(constant: TypedConstant) -> ModuleInterface {
         name: "a".into(),
         types: HashMap::new(),
         types_value_constructors: HashMap::new(),
-        unused_imports: Vec::new(),
         accessors: HashMap::new(),
         values: [(
             "one".into(),
@@ -63,6 +63,7 @@ fn constant_module(constant: TypedConstant) -> ModuleInterface {
         .into(),
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
     }
 }
 
@@ -92,10 +93,10 @@ fn empty_module() {
         types: HashMap::new(),
         types_value_constructors: HashMap::new(),
         values: HashMap::new(),
-        unused_imports: Vec::new(),
         accessors: HashMap::new(),
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -111,7 +112,6 @@ fn with_line_numbers() {
         types: HashMap::new(),
         types_value_constructors: HashMap::new(),
         values: HashMap::new(),
-        unused_imports: Vec::new(),
         accessors: HashMap::new(),
         line_numbers: LineNumbers::new(
             "const a = 1
@@ -119,6 +119,7 @@ fn with_line_numbers() {
         const c = 3",
         ),
         src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -134,7 +135,7 @@ fn module_with_private_type() {
         types: [(
             "ListIntType".into(),
             TypeConstructor {
-                typ: type_::list(type_::int()),
+                type_: type_::list(type_::int()),
                 publicity: Publicity::Private,
                 origin: Default::default(),
                 module: "the/module".into(),
@@ -146,32 +147,10 @@ fn module_with_private_type() {
         .into(),
         types_value_constructors: HashMap::new(),
         values: HashMap::new(),
-        unused_imports: Vec::new(),
         accessors: HashMap::new(),
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
-    };
-    assert_eq!(roundtrip(&module), module);
-}
-
-#[test]
-fn module_with_unused_import() {
-    let module = ModuleInterface {
-        warnings: vec![],
-        is_internal: false,
-        package: "some_package".into(),
-        origin: Origin::Src,
-        name: "a".into(),
-        types: HashMap::new(),
-        types_value_constructors: HashMap::new(),
-        unused_imports: vec![
-            SrcSpan { start: 0, end: 10 },
-            SrcSpan { start: 13, end: 42 },
-        ],
-        accessors: HashMap::new(),
-        values: HashMap::new(),
-        line_numbers: LineNumbers::new(""),
-        src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -187,7 +166,7 @@ fn module_with_app_type() {
         types: [(
             "ListIntType".into(),
             TypeConstructor {
-                typ: type_::list(type_::int()),
+                type_: type_::list(type_::int()),
                 publicity: Publicity::Public,
                 origin: Default::default(),
                 module: "the/module".into(),
@@ -199,10 +178,10 @@ fn module_with_app_type() {
         .into(),
         types_value_constructors: HashMap::new(),
         values: HashMap::new(),
-        unused_imports: Vec::new(),
         accessors: HashMap::new(),
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -218,7 +197,7 @@ fn module_with_fn_type() {
         types: [(
             "FnType".into(),
             TypeConstructor {
-                typ: type_::fn_(vec![type_::nil(), type_::float()], type_::int()),
+                type_: type_::fn_(vec![type_::nil(), type_::float()], type_::int()),
                 publicity: Publicity::Public,
                 origin: Default::default(),
                 module: "the/module".into(),
@@ -230,10 +209,10 @@ fn module_with_fn_type() {
         .into(),
         types_value_constructors: HashMap::new(),
         values: HashMap::new(),
-        unused_imports: Vec::new(),
         accessors: HashMap::new(),
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -249,7 +228,7 @@ fn module_with_tuple_type() {
         types: [(
             "TupleType".into(),
             TypeConstructor {
-                typ: type_::tuple(vec![type_::nil(), type_::float(), type_::int()]),
+                type_: type_::tuple(vec![type_::nil(), type_::float(), type_::int()]),
                 publicity: Publicity::Public,
                 origin: Default::default(),
                 module: "the/module".into(),
@@ -261,10 +240,10 @@ fn module_with_tuple_type() {
         .into(),
         types_value_constructors: HashMap::new(),
         values: HashMap::new(),
-        unused_imports: Vec::new(),
         accessors: HashMap::new(),
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -286,7 +265,7 @@ fn module_with_generic_type() {
             types: [(
                 "TupleType".into(),
                 TypeConstructor {
-                    typ: type_::tuple(vec![t1.clone(), t1.clone(), t2.clone()]),
+                    type_: type_::tuple(vec![t1.clone(), t1.clone(), t2.clone()]),
                     publicity: Publicity::Public,
                     origin: Default::default(),
                     module: "the/module".into(),
@@ -298,10 +277,10 @@ fn module_with_generic_type() {
             .into(),
             types_value_constructors: HashMap::new(),
             values: HashMap::new(),
-            unused_imports: Vec::new(),
             accessors: HashMap::new(),
             line_numbers: LineNumbers::new(""),
             src_path: "some_path".into(),
+            minimum_required_version: Version::new(0, 1, 0),
         }
     }
 
@@ -323,7 +302,7 @@ fn module_with_type_links() {
             types: [(
                 "SomeType".into(),
                 TypeConstructor {
-                    typ: type_,
+                    type_,
                     publicity: Publicity::Public,
                     origin: Default::default(),
                     module: "a".into(),
@@ -335,10 +314,10 @@ fn module_with_type_links() {
             .into(),
             types_value_constructors: HashMap::new(),
             values: HashMap::new(),
-            unused_imports: Vec::new(),
             accessors: HashMap::new(),
             line_numbers: LineNumbers::new(""),
             src_path: "some_path".into(),
+            minimum_required_version: Version::new(0, 1, 0),
         }
     }
 
@@ -360,7 +339,7 @@ fn module_with_type_constructor_documentation() {
             types: [(
                 "SomeType".into(),
                 TypeConstructor {
-                    typ: type_,
+                    type_,
                     publicity: Publicity::Public,
                     origin: Default::default(),
                     module: "a".into(),
@@ -372,10 +351,10 @@ fn module_with_type_constructor_documentation() {
             .into(),
             types_value_constructors: HashMap::new(),
             values: HashMap::new(),
-            unused_imports: Vec::new(),
             accessors: HashMap::new(),
             line_numbers: LineNumbers::new(""),
             src_path: "some_path".into(),
+            minimum_required_version: Version::new(0, 1, 0),
         }
     }
 
@@ -397,7 +376,7 @@ fn module_with_type_constructor_origin() {
             types: [(
                 "SomeType".into(),
                 TypeConstructor {
-                    typ: type_,
+                    type_,
                     publicity: Publicity::Public,
                     origin: SrcSpan {
                         start: 535,
@@ -412,10 +391,10 @@ fn module_with_type_constructor_origin() {
             .into(),
             types_value_constructors: HashMap::new(),
             values: HashMap::new(),
-            unused_imports: Vec::new(),
             accessors: HashMap::new(),
             line_numbers: LineNumbers::new(""),
             src_path: "some_path".into(),
+            minimum_required_version: Version::new(0, 1, 0),
         }
     }
 
@@ -442,11 +421,11 @@ fn module_type_to_constructors_mapping() {
             },
         )]
         .into(),
-        unused_imports: Default::default(),
         accessors: HashMap::new(),
         values: HashMap::new(),
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -461,7 +440,6 @@ fn module_fn_value() {
         origin: Origin::Src,
         name: "a".into(),
         types: HashMap::new(),
-        unused_imports: Vec::new(),
         types_value_constructors: HashMap::new(),
         accessors: HashMap::new(),
         values: [(
@@ -480,6 +458,8 @@ fn module_fn_value() {
                         start: 535,
                         end: 1100,
                     },
+                    external_erlang: None,
+                    external_javascript: None,
                     implementations: Implementations {
                         gleam: true,
                         uses_erlang_externals: false,
@@ -493,6 +473,7 @@ fn module_fn_value() {
         .into(),
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -507,7 +488,6 @@ fn deprecated_module_fn_value() {
         name: "a".into(),
         types: HashMap::new(),
         types_value_constructors: HashMap::new(),
-        unused_imports: Vec::new(),
         accessors: HashMap::new(),
         values: [(
             "one".into(),
@@ -527,6 +507,8 @@ fn deprecated_module_fn_value() {
                         start: 535,
                         end: 1100,
                     },
+                    external_erlang: None,
+                    external_javascript: None,
                     implementations: Implementations {
                         gleam: true,
                         uses_erlang_externals: false,
@@ -540,6 +522,7 @@ fn deprecated_module_fn_value() {
         .into(),
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -553,7 +536,6 @@ fn private_module_fn_value() {
         origin: Origin::Src,
         name: "a".into(),
         types: HashMap::new(),
-        unused_imports: Vec::new(),
         types_value_constructors: HashMap::new(),
         accessors: HashMap::new(),
         values: [(
@@ -572,6 +554,8 @@ fn private_module_fn_value() {
                         start: 535,
                         end: 1100,
                     },
+                    external_erlang: None,
+                    external_javascript: None,
                     implementations: Implementations {
                         gleam: true,
                         uses_erlang_externals: false,
@@ -585,6 +569,7 @@ fn private_module_fn_value() {
         .into(),
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -601,7 +586,6 @@ fn module_fn_value_regression() {
         name: "a/b/c".into(),
         types: HashMap::new(),
         types_value_constructors: HashMap::new(),
-        unused_imports: Vec::new(),
         accessors: HashMap::new(),
         values: [(
             "one".into(),
@@ -619,6 +603,8 @@ fn module_fn_value_regression() {
                         start: 52,
                         end: 1100,
                     },
+                    external_erlang: None,
+                    external_javascript: None,
                     implementations: Implementations {
                         gleam: true,
                         uses_erlang_externals: false,
@@ -632,6 +618,7 @@ fn module_fn_value_regression() {
         .into(),
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -647,7 +634,6 @@ fn module_fn_value_with_field_map() {
         name: "a".into(),
         types: HashMap::new(),
         types_value_constructors: HashMap::new(),
-        unused_imports: Vec::new(),
         accessors: HashMap::new(),
         values: [(
             "one".into(),
@@ -662,6 +648,8 @@ fn module_fn_value_with_field_map() {
                         arity: 20,
                         fields: [("ok".into(), 5), ("ko".into(), 7)].into(),
                     }),
+                    external_erlang: None,
+                    external_javascript: None,
                     module: "a".into(),
                     arity: 5,
                     location: SrcSpan { start: 2, end: 11 },
@@ -678,6 +666,7 @@ fn module_fn_value_with_field_map() {
         .into(),
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -695,7 +684,6 @@ fn record_value() {
         name: "a".into(),
         types: HashMap::new(),
         types_value_constructors: HashMap::new(),
-        unused_imports: Vec::new(),
         accessors: HashMap::new(),
         values: [(
             "one".into(),
@@ -721,6 +709,7 @@ fn record_value() {
         .into(),
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -738,7 +727,6 @@ fn record_value_with_field_map() {
         name: "a".into(),
         types: HashMap::new(),
         types_value_constructors: HashMap::new(),
-        unused_imports: Vec::new(),
         accessors: HashMap::new(),
         values: [(
             "one".into(),
@@ -767,6 +755,7 @@ fn record_value_with_field_map() {
         .into(),
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -783,7 +772,6 @@ fn accessors() {
         types: HashMap::new(),
         types_value_constructors: HashMap::new(),
         values: HashMap::new(),
-        unused_imports: Vec::new(),
         accessors: [
             (
                 "one".into(),
@@ -831,6 +819,7 @@ fn accessors() {
         .into(),
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -902,7 +891,7 @@ fn constant_tuple() {
 fn constant_list() {
     let module = constant_module(Constant::List {
         location: Default::default(),
-        typ: type_::int(),
+        type_: type_::int(),
         elements: vec![
             Constant::Int {
                 location: Default::default(),
@@ -949,7 +938,7 @@ fn constant_record() {
             },
         ],
         tag: "thetag".into(),
-        typ: type_::int(),
+        type_: type_::int(),
         field_map: None,
     });
 
@@ -967,7 +956,7 @@ fn constant_var() {
         location: Default::default(),
         module: None,
         name: "one_original".into(),
-        typ: type_::int(),
+        type_: type_::int(),
         constructor: Some(Box::from(ValueConstructor {
             publicity: Publicity::Public,
             deprecation: Deprecation::NotDeprecated,
@@ -996,7 +985,6 @@ fn constant_var() {
         name: "a".into(),
         types: HashMap::new(),
         types_value_constructors: HashMap::new(),
-        unused_imports: Vec::new(),
         accessors: HashMap::new(),
         values: [
             (
@@ -1045,6 +1033,7 @@ fn constant_var() {
         .into(),
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1217,7 +1206,7 @@ fn deprecated_type() {
         types: [(
             "ListIntType".into(),
             TypeConstructor {
-                typ: type_::list(type_::int()),
+                type_: type_::list(type_::int()),
                 publicity: Publicity::Public,
                 origin: Default::default(),
                 module: "the/module".into(),
@@ -1231,10 +1220,10 @@ fn deprecated_type() {
         .into(),
         types_value_constructors: HashMap::new(),
         values: HashMap::new(),
-        unused_imports: Vec::new(),
         accessors: HashMap::new(),
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
     };
     assert_eq!(roundtrip(&module), module);
 }
@@ -1249,7 +1238,6 @@ fn module_fn_value_with_external_implementations() {
         name: "a/b/c".into(),
         types: HashMap::new(),
         types_value_constructors: HashMap::new(),
-        unused_imports: Vec::new(),
         accessors: HashMap::new(),
         values: [(
             "one".into(),
@@ -1267,6 +1255,8 @@ fn module_fn_value_with_external_implementations() {
                         start: 52,
                         end: 1100,
                     },
+                    external_erlang: Some(("wibble".into(), "wobble".into())),
+                    external_javascript: Some(("wobble".into(), "wibble".into())),
                     implementations: Implementations {
                         gleam: false,
                         uses_erlang_externals: true,
@@ -1280,6 +1270,7 @@ fn module_fn_value_with_external_implementations() {
         .into(),
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1295,12 +1286,13 @@ fn internal_module_fn() {
         name: "a/b/c".into(),
         types: HashMap::new(),
         types_value_constructors: HashMap::new(),
-        unused_imports: Vec::new(),
         accessors: HashMap::new(),
         values: [(
             "one".into(),
             ValueConstructor {
-                publicity: Publicity::Internal,
+                publicity: Publicity::Internal {
+                    attribute_location: None,
+                },
                 deprecation: Deprecation::NotDeprecated,
                 type_: type_::int(),
                 variant: ValueConstructorVariant::ModuleFn {
@@ -1313,6 +1305,8 @@ fn internal_module_fn() {
                         start: 52,
                         end: 1100,
                     },
+                    external_erlang: Some(("wibble".into(), "wobble".into())),
+                    external_javascript: Some(("wobble".into(), "wibble".into())),
                     implementations: Implementations {
                         gleam: false,
                         uses_erlang_externals: true,
@@ -1326,6 +1320,60 @@ fn internal_module_fn() {
         .into(),
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
+    };
+
+    assert_eq!(roundtrip(&module), module);
+}
+
+#[test]
+fn internal_annotated_module_fn() {
+    let module = ModuleInterface {
+        warnings: vec![],
+        is_internal: false,
+        package: "some_package".into(),
+        origin: Origin::Src,
+        name: "a/b/c".into(),
+        types: HashMap::new(),
+        types_value_constructors: HashMap::new(),
+        accessors: HashMap::new(),
+        values: [(
+            "one".into(),
+            ValueConstructor {
+                publicity: Publicity::Internal {
+                    attribute_location: Some(SrcSpan {
+                        start: 11,
+                        end: 111,
+                    }),
+                },
+                deprecation: Deprecation::NotDeprecated,
+                type_: type_::int(),
+                variant: ValueConstructorVariant::ModuleFn {
+                    documentation: Some("wabble!".into()),
+                    name: "one".into(),
+                    field_map: None,
+                    module: "a".into(),
+                    arity: 5,
+                    location: SrcSpan {
+                        start: 52,
+                        end: 1100,
+                    },
+                    external_erlang: Some(("wibble".into(), "wobble".into())),
+                    external_javascript: Some(("wobble".into(), "wibble".into())),
+                    implementations: Implementations {
+                        gleam: false,
+                        uses_erlang_externals: true,
+                        uses_javascript_externals: true,
+                        can_run_on_erlang: true,
+                        can_run_on_javascript: true,
+                    },
+                },
+            },
+        )]
+        .into(),
+        line_numbers: LineNumbers::new(""),
+        src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
     };
 
     assert_eq!(roundtrip(&module), module);
@@ -1361,11 +1409,11 @@ fn type_variable_ids_in_constructors_are_shared() {
                 }],
             },
         )]),
-        unused_imports: Vec::new(),
         accessors: HashMap::new(),
         values: [].into(),
         line_numbers: LineNumbers::new(""),
         src_path: "some_path".into(),
+        minimum_required_version: Version::new(0, 1, 0),
     };
 
     let expected = HashMap::from([(
