@@ -15,7 +15,7 @@ fn print(type_: Arc<Type>) -> String {
 fn custom_bool() -> Arc<Type> {
     Arc::new(Type::Named {
         publicity: Publicity::Public,
-        package: "foo".into(),
+        package: "wibble".into(),
         module: "one/two".into(),
         name: "Bool".into(),
         args: vec![],
