@@ -1,5 +1,8 @@
 use super::*;
-use crate::{assert_no_warnings, assert_warning, assert_warnings_with_imports};
+use crate::{
+    assert_no_warnings, assert_warning, assert_warnings_with_gleam_version,
+    assert_warnings_with_imports,
+};
 
 #[test]
 fn unknown_label() {
@@ -787,7 +790,7 @@ pub fn main() {
   x
 }"#;
     let warnings = VectorWarningEmitterIO::default();
-    _ = compile_module("test_module", src, Some(Arc::new(warnings.clone())), vec![]).unwrap_err();
+    _ = compile_module("test_module", src, Some(Rc::new(warnings.clone())), vec![]).unwrap_err();
     assert!(warnings.take().is_empty());
 }
 
@@ -1332,6 +1335,23 @@ pub fn main() {
     );
 }
 
+#[test]
+fn unused_pipeline_ending_with_variant_raises_a_warning_2() {
+    assert_warning!(
+        ("wibble", "pub type Wibble { Wibble(Int) }"),
+        r#"
+import wibble
+
+pub fn wobble(a) { a }
+
+pub fn main() {
+  1 |> wobble |> wibble.Wibble
+  1
+}
+"#
+    );
+}
+
 #[test]
 fn unused_pipeline_not_ending_with_variant_raises_no_warnings() {
     assert_no_warnings!(
@@ -1347,6 +1367,21 @@ pub fn main() {
     );
 }
 
+#[test]
+fn unused_module_select_constructor() {
+    assert_warning!(
+        ("wibble", "pub type Wibble { Wibble(Int) }"),
+        r#"
+import wibble
+
+pub fn main() {
+  wibble.Wibble(1)
+  1
+}
+"#
+    );
+}
+
 /*
 
 TODO: These tests are commented out until we figure out a better way to deal
@@ -1986,6 +2021,97 @@ fn deprecated_list_pattern_syntax_1() {
     );
 }
 
+// https://github.com/gleam-lang/gleam/issues/3473
+#[test]
+fn deprecated_record_pattern_syntax() {
+    assert_warning!(
+        r#"
+pub type Wibble {
+  Wibble(one: Int, two: Int)
+}
+
+pub fn main() {
+  let wibble = Wibble(one: 1, two: 2)
+  case wibble {
+    Wibble(one: one ..) -> one
+  }
+}
+"#
+    );
+}
+
+#[test]
+fn deprecated_record_pattern_syntax_with_no_labels() {
+    assert_warning!(
+        r#"
+pub type Wibble {
+  Wibble(one: Int, two: Int)
+}
+
+pub fn main() {
+  let wibble = Wibble(one: 1, two: 2)
+  case wibble {
+    Wibble(one ..) -> one
+  }
+}
+"#
+    );
+}
+
+#[test]
+fn deprecated_record_pattern_syntax_with_label_shorthand() {
+    assert_warning!(
+        r#"
+pub type Wibble {
+  Wibble(one: Int, two: Int)
+}
+
+pub fn main() {
+  let wibble = Wibble(one: 1, two: 2)
+  case wibble {
+    Wibble(one: ..) -> one
+  }
+}
+"#
+    );
+}
+
+#[test]
+fn deprecated_record_pattern_syntax_has_no_warning_if_everything_is_discarded() {
+    assert_no_warnings!(
+        r#"
+pub type Wibble {
+  Wibble(one: Int, two: Int)
+}
+
+pub fn main() {
+  let wibble = Wibble(one: 1, two: 2)
+  case wibble {
+    Wibble(..) -> 1
+  }
+}
+"#
+    );
+}
+
+#[test]
+fn deprecated_record_pattern_syntax_has_no_warning_if_there_is_a_comma_before_spread() {
+    assert_no_warnings!(
+        r#"
+pub type Wibble {
+  Wibble(one: Int, two: Int)
+}
+
+pub fn main() {
+  let wibble = Wibble(one: 1, two: 2)
+  case wibble {
+    Wibble(one: one, ..) -> one
+  }
+}
+"#
+    );
+}
+
 #[test]
 fn unused_label_shorthand_pattern_arg() {
     assert_warning!(
@@ -2014,3 +2140,282 @@ pub fn main() {
 "#
     );
 }
+
+#[test]
+fn internal_annotation_on_constant_requires_v1_1() {
+    assert_warnings_with_gleam_version!(
+        Range::higher_than(Version::new(1, 0, 0)),
+        "
+@internal
+pub const wibble = 1
+",
+    );
+}
+
+#[test]
+fn internal_annotation_on_type_requires_v1_1() {
+    assert_warnings_with_gleam_version!(
+        Range::higher_than(Version::new(1, 0, 0)),
+        "
+@internal
+pub type Wibble
+",
+    );
+}
+
+#[test]
+fn internal_annotation_on_function_requires_v1_1() {
+    assert_warnings_with_gleam_version!(
+        Range::higher_than(Version::new(1, 0, 0)),
+        "
+@internal
+pub fn wibble() { Nil }
+",
+    );
+}
+
+#[test]
+fn nested_tuple_access_requires_v1_1() {
+    assert_warnings_with_gleam_version!(
+        Range::higher_than(Version::new(1, 0, 0)),
+        "
+pub fn main() {
+  let tuple = #(1, #(1, 1))
+  tuple.1.0
+}
+",
+    );
+}
+
+#[test]
+fn javascript_external_module_with_at_requires_v1_2() {
+    assert_warnings_with_gleam_version!(
+        Range::higher_than(Version::new(1, 0, 0)),
+        "
+@external(javascript, \"module@module\", \"func\")
+pub fn main() { Nil }
+",
+    );
+}
+
+#[test]
+fn int_plus_in_guards_requires_v1_3() {
+    assert_warnings_with_gleam_version!(
+        Range::higher_than(Version::new(1, 0, 0)),
+        "
+pub fn main() {
+  case Nil {
+    _ if 1 + 1 == 2 -> Nil
+    _ -> Nil
+  }
+}
+",
+    );
+}
+
+#[test]
+fn float_plus_in_guards_requires_v1_3() {
+    assert_warnings_with_gleam_version!(
+        Range::higher_than(Version::new(1, 0, 0)),
+        "
+pub fn main() {
+  case Nil {
+    _ if 1.0 +. 1.0 == 2.0 -> Nil
+    _ -> Nil
+  }
+}
+",
+    );
+}
+
+#[test]
+fn int_minus_in_guards_requires_v1_3() {
+    assert_warnings_with_gleam_version!(
+        Range::higher_than(Version::new(1, 0, 0)),
+        "
+pub fn main() {
+  case Nil {
+    _ if 1 - 1 == 0 -> Nil
+    _ -> Nil
+  }
+}
+",
+    );
+}
+
+#[test]
+fn float_minus_in_guards_requires_v1_3() {
+    assert_warnings_with_gleam_version!(
+        Range::higher_than(Version::new(1, 0, 0)),
+        "
+pub fn main() {
+  case Nil {
+    _ if 1.0 -. 1.0 == 0.0 -> Nil
+    _ -> Nil
+  }
+}
+",
+    );
+}
+
+#[test]
+fn int_multiplication_in_guards_requires_v1_3() {
+    assert_warnings_with_gleam_version!(
+        Range::higher_than(Version::new(1, 0, 0)),
+        "
+pub fn main() {
+  case Nil {
+  _ if 1 * 1 == 0 -> Nil
+  _ -> Nil
+  }
+}
+",
+    );
+}
+
+#[test]
+fn float_multiplication_in_guards_requires_v1_3() {
+    assert_warnings_with_gleam_version!(
+        Range::higher_than(Version::new(1, 0, 0)),
+        "
+pub fn main() {
+  case Nil {
+  _ if 1.0 *. 1.0 == 0.0 -> Nil
+  _ -> Nil
+  }
+}
+",
+    );
+}
+
+#[test]
+fn int_divide_in_guards_requires_v1_3() {
+    assert_warnings_with_gleam_version!(
+        Range::higher_than(Version::new(1, 0, 0)),
+        "
+pub fn main() {
+  case Nil {
+  _ if 1 / 1 == 0 -> Nil
+  _ -> Nil
+  }
+}
+",
+    );
+}
+
+#[test]
+fn float_divide_in_guards_requires_v1_3() {
+    assert_warnings_with_gleam_version!(
+        Range::higher_than(Version::new(1, 0, 0)),
+        "
+pub fn main() {
+  case Nil {
+  _ if 1.0 /. 1.0 == 0.0 -> Nil
+  _ -> Nil
+  }
+}
+",
+    );
+}
+
+#[test]
+fn int_remainder_in_guards_requires_v1_3() {
+    assert_warnings_with_gleam_version!(
+        Range::higher_than(Version::new(1, 0, 0)),
+        "
+pub fn main() {
+  case Nil {
+  _ if 1 % 1 == 0 -> Nil
+  _ -> Nil
+  }
+}
+",
+    );
+}
+
+#[test]
+fn label_shorthand_in_constand_requires_v1_4() {
+    assert_warnings_with_gleam_version!(
+        Range::higher_than(Version::new(1, 0, 0)),
+        "
+pub type Wibble { Wibble(wibble: Int) }
+
+pub const wibble = 1
+pub const wobble = Wibble(wibble:)
+",
+    );
+}
+
+#[test]
+fn label_shorthand_in_call_requires_v1_4() {
+    assert_warnings_with_gleam_version!(
+        Range::higher_than(Version::new(1, 0, 0)),
+        "
+pub type Wibble { Wibble(wibble: Int) }
+
+pub fn main() {
+  let wibble = 1
+  Wibble(wibble:)
+}
+",
+    );
+}
+
+#[test]
+fn label_shorthand_in_pattern_requires_v1_4() {
+    assert_warnings_with_gleam_version!(
+        Range::higher_than(Version::new(1, 0, 0)),
+        "
+pub type Wibble { Wibble(wibble: Int) }
+
+pub fn main(wibble) {
+  case wibble {
+    Wibble(wibble:) -> wibble
+  }
+}
+",
+    );
+}
+
+#[test]
+fn constant_string_concatenation_requires_v1_4() {
+    assert_warnings_with_gleam_version!(
+        Range::higher_than(Version::new(1, 0, 0)),
+        "pub const string = \"wibble\" <> \"wobble\""
+    );
+}
+
+#[test]
+fn missing_utf_8_option_in_bit_array_segment_requires_v1_5() {
+    assert_warnings_with_gleam_version!(
+        Range::higher_than(Version::new(1, 0, 0)),
+        "
+pub fn main() {
+  <<\"hello\">>
+}
+",
+    );
+}
+
+#[test]
+fn missing_utf_8_option_in_bit_array_constant_segment_requires_v1_5() {
+    assert_warnings_with_gleam_version!(
+        Range::higher_than(Version::new(1, 0, 0)),
+        "pub const bits = <<\"hello\">>"
+    );
+}
+
+#[test]
+fn missing_utf_8_option_in_bit_array_pattern_segment_requires_v1_5() {
+    assert_warnings_with_gleam_version!(
+        Range::higher_than(Version::new(1, 0, 0)),
+        "
+pub fn main(a) {
+  case a {
+    <<\"hello\">> -> Nil
+    _ -> Nil
+  }
+}
+",
+    );
+}
