@@ -21,7 +21,8 @@ use regex::Regex;
 use std::{
     collections::{HashMap, HashSet},
     fmt::Write,
-    sync::Arc,
+    rc::Rc,
+    sync::OnceLock,
 };
 
 use camino::{Utf8Path, Utf8PathBuf};
@@ -48,7 +49,7 @@ pub fn prepare(path: &str) -> String {
     let ids = gleam_core::uid::UniqueIdGenerator::new();
     let mut modules = im::HashMap::new();
     let warnings = VectorWarningEmitterIO::default();
-    let warning_emitter = WarningEmitter::new(Arc::new(warnings.clone()));
+    let warning_emitter = WarningEmitter::new(Rc::new(warnings.clone()));
     let filesystem = to_in_memory_filesystem(&root);
     let initial_files = filesystem.paths();
     let root = Utf8PathBuf::from("");
@@ -103,6 +104,8 @@ fn normalise_diagnostic(text: &str) -> String {
         .replace('\\', "/")
 }
 
+static FILE_LINE_REGEX: OnceLock<Regex> = OnceLock::new();
+
 #[derive(Debug)]
 pub struct TestCompileOutput {
     files: HashMap<Utf8PathBuf, Content>,
@@ -113,15 +116,27 @@ impl TestCompileOutput {
     pub fn as_overview_text(&self) -> String {
         let mut buffer = String::new();
         for (path, content) in self.files.iter().sorted_by(|a, b| a.0.cmp(b.0)) {
+            let normalised_path = path.as_str().replace('\\', "/");
             buffer.push_str("//// ");
-            buffer.push_str(&path.as_str().replace('\\', "/"));
+            buffer.push_str(&normalised_path);
             buffer.push('\n');
 
             let extension = path.extension();
             match content {
                 _ if extension == Some("cache") => buffer.push_str("<.cache binary>"),
                 Content::Binary(data) => write!(buffer, "<{} byte binary>", data.len()).unwrap(),
-                Content::Text(text) => buffer.push_str(text),
+                Content::Text(text) => {
+                    let text = FILE_LINE_REGEX
+                        .get_or_init(|| {
+                            Regex::new(r#"-file\("([^"]+)", (\d+)\)\."#).expect("Invalid regex")
+                        })
+                        .replace_all(text, |caps: &regex::Captures| {
+                            let path = caps.get(1).expect("file path").as_str().replace("\\", "/");
+                            let line_number = caps.get(2).expect("line number").as_str();
+                            format!("-file(\"{}\", {}).", path, line_number)
+                        });
+                    buffer.push_str(&text)
+                }
             };
             buffer.push('\n');
             buffer.push('\n');
